// This is the main code, it should run on the main processor.
// Read PPM signals from 2 channels of an RC reciever and convert the values to PWM in either direction.
// digital pins 5 & 9 control motor1, digital pins 6 & 10 control motor2. 
// DP 12 and 13 are neutral indicator lights. 
// DP 2 and 3 are inputs from the R/C receiver. 
// All analog pins are open. 
// 
// JDW 2009
// www.rediculouslygoodlooking.com



int motor1_a = 5;
int motor1_b = 9;

int motor2_a = 6;
int motor2_b = 10;


int ledPin1 = 12;
int ledPin2 = 13;
int ledPin3 = 14;

int ppm1 = 2; 
int ppm2 = 3;


unsigned long servo1_start;
volatile unsigned servo1_val; 
int adj_val1;  
int servo1_Ready;

unsigned long servo2_start;
volatile unsigned servo2_val; 
int adj_val2;  
int servo2_Ready;

int deadband_high = 275; 
int deadband_low = 235;  

int pwm_ceiling = 256; 
int pwm_floor = 255;  

int low1 = 1000;
int high1 = 2000;
int low2 = 1000;
int high2 = 2000;

void setup() {

  Serial.begin(9600);

  //motor1 pins
  pinMode(motor1_a, OUTPUT);
  pinMode(motor1_b, OUTPUT);

  pinMode(motor2_a, OUTPUT);
  pinMode(motor2_b, OUTPUT);

  //led's
  pinMode(ledPin1, OUTPUT);
  pinMode(ledPin2, OUTPUT);
  pinMode(ledPin3, OUTPUT);

  //PPM inputs from RC receiver
  pinMode(ppm1, INPUT); //Pin 2 as input
  pinMode(ppm2, INPUT); //Pin 3 as input

  attachInterrupt(0, ISR1_begin, RISING);    // catch interrupt 0 (digital pin 2) going HIGH and send to rc1()
  attachInterrupt(1, ISR2_begin, RISING);    // catch interrupt 1 (digital pin 3) going HIGH and send to rc2()


}


void ISR1_begin() {

  servo1_start = micros();

  detachInterrupt(0);
  
  attachInterrupt(0, ISR1_end, FALLING);
  
}

void ISR1_end() {
 
 servo1_val = micros() - servo1_start;
  
 detachInterrupt(0);
  
 attachInterrupt(0, ISR1_begin, RISING); 
 
 if (servo1_val < 600 || servo1_val > 2400) {
  servo1_Ready = false; 
 }
 else {
  servo1_Ready = true; 
 }
}


void ISR2_begin() {

  servo2_start = micros();

  detachInterrupt(1);
  
  attachInterrupt(1, ISR2_end, FALLING);
  
}

void ISR2_end() {
 
 servo2_val = micros() - servo2_start;
  
 detachInterrupt(1);
  
 attachInterrupt(1, ISR2_begin, RISING); 
 
 if (servo2_val < 600 || servo2_val > 2400) {
  servo2_Ready = false; 
 }
 else {
  servo2_Ready = true; 
 }
}


void loop() {
	  


    if (servo1_Ready)
    {

      
      servo1_Ready = false;  
      adj_val1 = map(constrain(servo1_val, 600, 2400), low1, high1, 0, 511); 
      constrain(adj_val1, 0, 511);


      if (adj_val1 > deadband_high) {
        //Forward
        digitalWrite(motor1_b, LOW);
        analogWrite(motor1_a, adj_val1 - pwm_ceiling); //write adj_val1 to motor1
        //    analogWrite(motor1_a, 511 - adj_val1); // .... inverted formula
        digitalWrite(ledPin1, LOW);
      }
      else if (adj_val1 < deadband_low) {
        //REVERSE
        digitalWrite(motor1_a, LOW);
        analogWrite(motor1_b, pwm_floor - adj_val1); //write adj_val1 to motor1
        //    analogWrite(motor1_b, adj_val1); // .... inverted formula
        digitalWrite(ledPin1, LOW);
      }
      else {
        //STOP
        digitalWrite(motor1_a, LOW);
        digitalWrite(motor1_b, LOW);
        digitalWrite(ledPin1, HIGH); // turn on neutral light, turn motor pins off
      }
      
    }
    


    if (servo2_Ready)
    {
      servo2_Ready = false;

      adj_val2 = map(constrain(servo2_val, 600, 2400), low2, high2, 0, 511); 
      constrain(adj_val2, 0, 511);
      
      
      if (adj_val2 > deadband_high) {
        //Forward
        digitalWrite(motor2_b, LOW);
        analogWrite(motor2_a, adj_val2 - pwm_ceiling); //write adj_val1 to motor1
        //    analogWrite(motor2_a, 511 - adj_val2); // .... inverted formula
        digitalWrite(ledPin2, LOW);
      }
      else if (adj_val2 < deadband_low) {
        //REVERSE
        digitalWrite(motor2_a, LOW);
        analogWrite(motor2_b, pwm_floor - adj_val2); //write adj_val1 to motor1
        //    analogWrite(motor2_b, adj_val2); // .... inverted formula
        digitalWrite(ledPin2, LOW);
      }
      else {
        //STOP
        digitalWrite(motor2_a, LOW);
        digitalWrite(motor2_b, LOW);
        digitalWrite(ledPin2, HIGH); // turn on neutral light, turn motor pins off
      }
    }



    //print values
    Serial.print("ch1:  ");
    Serial.print(adj_val1);
    Serial.print("  ");
    Serial.print("rc1:  ");
    Serial.print(servo1_val);
    Serial.print("  ");
    Serial.print("ch2:  ");
    Serial.print(adj_val2);
    Serial.print("  ");
    Serial.print("rc2:  ");
    Serial.print(servo2_val);
    Serial.println("  ");    


 
}
